VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "FileAssociation"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
'A class to read and make file association registry entries
'
Option Explicit

Private mExtension As String
Private mFileClass As FileClass
Private mLastError As String

Public Enum LoadAssociationResult
  LOAD_ASOC_SUCCESS = 0
  LOAD_ASOC_NO_FILE_CLASS = 1
  LOAD_ASOC_NO_SHELL_COMMANDS = 2
  LOAD_ASOC_INVALID_PARAMETER = 3
  LOAD_ASOC_NOT_REGISTERED = 4
End Enum
'
'The extension is subject to checking.
'
Public Property Let Extension(lExtension As String)
  mExtension = GoodExtension(lExtension)
End Property
'
'All the string properties can be read
'
Public Property Get Extension() As String
Extension = mExtension
End Property
Public Property Get LastError() As String
    LastError = mLastError
End Property
Public Property Get FileClass() As FileClass
Set FileClass = mFileClass
End Property
Public Property Set FileClass(Value As FileClass)
Set mFileClass = Value
End Property
'
'Completely clear the association
'
Public Sub Clear()

  mExtension = ""
  Set mFileClass = New FileClass

End Sub
'
'Convert a string to an extension with a dot first, or empty
'
Public Function GoodExtension(lExtension As String) As String

Dim Pos As Long
Dim lResult As String

'Check not empty or just dot
lResult = OriginalString(lExtension)
If Len(lResult) = 0 Or lResult = "." Then
    GoodExtension = ""
    Exit Function
End If

'Check there are no spaces
Pos = InStr(lResult, " ")
If Pos > 0 Then
    GoodExtension = ""
    Exit Function
End If

'Add a leading dot if necessary
If Left$(lResult, 1) = "." Then
  GoodExtension = lResult
Else
  GoodExtension = "." & lResult
End If

End Function
'
'Given a file extension, fill in as many class members as possible from the registry
'Default the non-verb members whether or not any registry entries were found,
'unless the parameter is invalid
'
Public Function LoadFromRegistry(Extension As String) As LoadAssociationResult

Dim lClassName As String
Dim lExtension As String

lExtension = GoodExtension(Extension)

If Len(lExtension) = 0 Then
  mLastError = "The extension is not valid"
  LoadFromRegistry = LOAD_ASOC_INVALID_PARAMETER
  Exit Function
End If

'Clear all the details
Call Clear
mExtension = lExtension

'Get the Extension default value
If RegReadDefaultStringValue(HKEY_CLASSES_ROOT, mExtension, lClassName) <> ERROR_SUCCESS Then
  mLastError = "The extension """ & mExtension & """ is not registered"
  LoadFromRegistry = LOAD_ASOC_NOT_REGISTERED
  Exit Function
End If

'Check that the File Class exists
If lClassName = "" Then
  mLastError = "The File Class name is empty"
  LoadFromRegistry = LOAD_ASOC_NO_FILE_CLASS
  Exit Function
End If

'Now load all the File Class details
If Not mFileClass.LoadFromRegistry(lClassName) Then
  mLastError = mFileClass.LastError
  LoadFromRegistry = LOAD_ASOC_NO_FILE_CLASS
  Exit Function
End If

LoadFromRegistry = LOAD_ASOC_SUCCESS

End Function
'
'Write the association to the registry if all is in order
'
Public Function SaveToRegistry() As Boolean

Dim Res As Long

'Assume the worst
SaveToRegistry = False

'Check that all the necessary information is present
If Len(mExtension) = 0 Then
  mLastError = "You have not specified a file extension"
  Exit Function
End If

If Len(mFileClass.ClassName) = 0 Then
  If MsgBox("The class name is empty. Saving now will remove all associations. " & _
    "Do you wish to proceed?", vbYesNo) = vbNo Then
      SaveToRegistry = True
      Exit Function
  End If
End If

'Write the extension registry key
Res = RegWriteDefaultStringValue(HKEY_CLASSES_ROOT, mExtension, mFileClass.ClassName)
If Res <> ERROR_SUCCESS Then
    mLastError = "Cannot write the Extension and File Class to the Registry. " & _
    "You may not have the necessary permission" & SysErrorString(Res)
  Exit Function
End If

'Write the file class if the name exists
If Len(mFileClass.ClassName) <> 0 Then
  If Not mFileClass.SaveToRegistry() Then
    mLastError = mFileClass.LastError
    Exit Function
  End If
End If

SaveToRegistry = True

End Function
'
'Create a string which lists all the associated commands
'
Public Function Enumerate() As String

Enumerate = "Extension: " & mExtension & vbCrLf & mFileClass.Enumerate

End Function
Private Sub Class_Initialize()
    Call Clear
End Sub

Private Sub Class_Terminate()
  Set mFileClass = Nothing
End Sub
