VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "FileClass"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
'A class to read and make File Class registry entries
'
Option Explicit

Private mClassName As String
Private mDescription As String
Private mLastError As String
Private mDefaultVerb As String
Private mDefaultIconFile As String
Private mIconIndex As Long

Private mVerbs As Collection
'
'The user may modify the Class name and description directly
'
Public Property Let ClassName(lClassName As String)
mClassName = OriginalString(lClassName)
End Property
Public Property Let Description(lDescription As String)
mDescription = OriginalString(lDescription)
End Property
Public Property Let DefaultVerb(lVerb As String)
mDefaultVerb = OriginalString(lVerb)
End Property
Public Property Get ClassName() As String
ClassName = mClassName
End Property
Public Property Get Description() As String
Description = mDescription
End Property
Public Property Get DefaultVerb() As String
DefaultVerb = mDefaultVerb
End Property
Public Property Get LastError() As String
    LastError = mLastError
End Property
Public Property Get DefaultIconFile() As String
    DefaultIconFile = mDefaultIconFile
End Property
Public Property Let DefaultIconFile(Val As String)
    mDefaultIconFile = Val
End Property
Public Property Get IconIndex() As Long
    IconIndex = mIconIndex
End Property
Public Property Let IconIndex(Val As Long)
    mIconIndex = Val
End Property
'
'Return a particular verb from the collection. Default member.
'
Public Function Verb(Index As Variant) As ExecVerb
Attribute Verb.VB_UserMemId = 0

Dim lKey As String

On Error GoTo NotPresent

'If the variant is a string, get a lower case version of the key
If TypeName(Index) = "String" Then
  lKey = StrConv(Index, vbLowerCase)
  Set Verb = mVerbs(lKey)
Else
  Set Verb = mVerbs(Index)
End If
  
On Error GoTo 0

Exit Function

NotPresent:
On Error GoTo 0
Set Verb = Nothing
End Function
'
'NewEnum
'=======
'
'Standard routine to return next item in a For Each..Next iteration
'
'NB. Must set the Procedure ID to -4 and check the "Hide this member"
'    checkbox (Advanced settings)
'
Public Property Get NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"

  'Return the next item in the iteration
  Set NewEnum = mVerbs.[_NewEnum]
  
End Property
'
'Return the number of verbs in the collection
'
Public Function VerbCount() As Integer
  VerbCount = mVerbs.Count
End Function
'
'Completely clear the association
'
Public Sub Clear()

  Set mVerbs = New Collection
  
  mClassName = ""
  mDescription = ""
  mLastError = ""
  mDefaultVerb = ""
  mDefaultIconFile = ""
  
End Sub
'
'Given a file extension, fill in as many class members as possible from the registry
'Return False on an error, LastError will contain the reason
'
Public Function LoadFromRegistry(ClassName As String) As Boolean

Dim hkeyShell As Long
Dim KeyIndex As Long
Dim NextVerb As String
Dim VerbObject As ExecVerb
Dim lKey As String
Dim CommandString As String
Dim lDescription As String
Dim Res As Long
Dim stIconIndexTemp As String

'Assume the worst
LoadFromRegistry = False

If Len(OriginalString(ClassName)) = 0 Then
  mLastError = "Class name is empty"
  Exit Function
End If

'Now open the ClassName key to get the shell key.
If RegReadDefaultStringValue(HKEY_CLASSES_ROOT, ClassName, lDescription) <> ERROR_SUCCESS Then
  mLastError = "The Class name is not regestered"
  Exit Function
End If

LoadFromRegistry = True

'Clear all the details, there is a class of the given name
Call Clear
mClassName = ClassName
mDescription = lDescription

'Get a key for the DefaultIcon sub-key. Not an error if none
Res = RegOpenKeyEx(HKEY_CLASSES_ROOT, ClassName & "\DefaultIcon", 0, KEY_READ, hkeyShell)
If Res = ERROR_SUCCESS Then
    'There may or may not be a default value, so ignore error
    Call RegReadDefaultStringValue(hkeyShell, "", mDefaultIconFile)
    
    If mDefaultIconFile <> "" Then
        Res = InStr(mDefaultIconFile, ",")
        If Res > 0 Then
            stIconIndexTemp = Right$(mDefaultIconFile, Len(mDefaultIconFile) - Res)
            Do While Not IsNumeric(Right$(stIconIndexTemp, 1))
             stIconIndexTemp = Left$(stIconIndexTemp, Len(stIconIndexTemp) - 1)
            Loop
            mIconIndex = CLng(stIconIndexTemp)
            mDefaultIconFile = Left$(mDefaultIconFile, Res - 1)
        Else
            mIconIndex = 0
        End If
    End If
    
    Call RegCloseKey(hkeyShell)
Else
    mDefaultIconFile = ""
End If

'Now get a key for the Shell sub-key. Not an error if none
Res = RegOpenKeyEx(HKEY_CLASSES_ROOT, ClassName & "\Shell", 0, KEY_READ, hkeyShell)
If Res <> ERROR_SUCCESS Then Exit Function

'There may or may not be a default verb, so ignore error
Call RegReadDefaultStringValue(hkeyShell, "", mDefaultVerb)

For KeyIndex = 0 To 100
  NextVerb = String(100, Chr$(0))
  Res = RegEnumKey(hkeyShell, KeyIndex, NextVerb, Len(NextVerb))
  If Res <> ERROR_SUCCESS Then
     Exit For
  End If

  'Strip a null terminator
  NextVerb = VBstring(NextVerb)

  'Create ExecVerb object and add to the collection
  Set VerbObject = New ExecVerb
  VerbObject.Verb = NextVerb
  
  'Look for the command, if any
  If RegReadDefaultStringValue(hkeyShell, NextVerb & "\Command", CommandString) = ERROR_SUCCESS Then
    VerbObject.Command = CommandString
  End If
  
  If RegReadDefaultStringValue(hkeyShell, NextVerb & "\Ddeexec", CommandString) = ERROR_SUCCESS Then
    VerbObject.Ddeexec = CommandString
  End If
  
  If RegReadDefaultStringValue(hkeyShell, NextVerb & "\Ddeexec\ifexec", CommandString) = ERROR_SUCCESS Then
    VerbObject.IfExecDde = CommandString
  End If
  
  If RegReadDefaultStringValue(hkeyShell, NextVerb & "\Ddeexec\Application", CommandString) = ERROR_SUCCESS Then
    VerbObject.Application = CommandString
  End If
  
  If RegReadDefaultStringValue(hkeyShell, NextVerb & "\Ddeexec\Topic", CommandString) = ERROR_SUCCESS Then
    VerbObject.Topic = CommandString
  End If
  
  'Make a lower case key for the verb
  lKey = StrConv(NextVerb, vbLowerCase)
  Call mVerbs.Add(VerbObject, lKey)
Next KeyIndex

Call RegCloseKey(hkeyShell)

End Function
'
'Given a file extension, fill in as many class members as possible from the registry
'Default the non-verb members whether or not any registry entries were found,
'unless the parameter is invalid
'Return the system error code
'
Public Function SaveToRegistry() As Boolean

Dim hkeyShell As Long
Dim KeyIndex As Long
Dim NextVerb As String
Dim VerbObject As ExecVerb
Dim lKey As String
Dim Res As Long
Dim CommandString As String
Dim ReturnValueSet As Boolean
Dim Disposition As Long

'Assume the worst
SaveToRegistry = False

'Check that all the necessary information is present
If Len(mClassName) = 0 Then
  mLastError = "You must specify a File Class name to save Action Verbs"
  Exit Function
End If
If mVerbs.Count = 0 Then
  mLastError = "There are no Action Verbs for the File Class"
  Exit Function
End If

'From here, the error exits are all because we cannot write the registry
mLastError = "Unable to write to the Registry, you probably don't have the necessary permission"

'Write the description
Res = RegWriteDefaultStringValue(HKEY_CLASSES_ROOT, mClassName, mDescription)
If Res <> ERROR_SUCCESS Then
    mLastError = "Could not write the class description" & SysErrorString(Res)
    Exit Function
End If

'Rewrite the default icon string
'This may fail because it does not exist or we don't have permission.
'If its a permission problem creating the key will also fail, so ignore errors here
Res = DeleteKey(HKEY_CLASSES_ROOT, mClassName & "\DefaultIcon")

'Only write the default icon key if there is one
If mDefaultIconFile <> "" Then
    Res = RegCreateKeyEx(HKEY_CLASSES_ROOT, mClassName & "\DefaultIcon", 0, "", REG_OPTION_NON_VOLATILE, _
                KEY_ALL_ACCESS, 0, hkeyShell, Disposition)
    If Res <> ERROR_SUCCESS Then
        mLastError = "Could not create the Default Icon key" & SysErrorString(Res)
        Exit Function
    End If
    
    Res = RegWriteDefaultStringValue(hkeyShell, "", mDefaultIconFile & "," & CStr(mIconIndex))
    Call RegCloseKey(hkeyShell)
    If Res <> ERROR_SUCCESS Then
        mLastError = "Could not write the default verb" & SysErrorString(Res)
        Exit Function
    End If
End If

'Delete the key then re-create it to remove the previous set of verbs
'This may fail because it does not exist or we don't have permission.
'If its a permission problem creating the key will also fail
Res = DeleteKey(HKEY_CLASSES_ROOT, mClassName & "\Shell")
If Res <> ERROR_SUCCESS Then
    mLastError = "Could not delete the Shell key" & SysErrorString(Res)
    Exit Function
End If

Res = RegCreateKeyEx(HKEY_CLASSES_ROOT, mClassName & "\Shell", 0, "", REG_OPTION_NON_VOLATILE, _
            KEY_ALL_ACCESS, 0, hkeyShell, Disposition)
If Res <> ERROR_SUCCESS Then
    mLastError = "Could not create the Shell key" & SysErrorString(Res)
    Exit Function
End If

'There may or may not be a default verb
Res = RegWriteDefaultStringValue(hkeyShell, "", mDefaultVerb)
If Res <> ERROR_SUCCESS Then
    mLastError = "Could not write the default verb" & SysErrorString(Res)
    Exit Function
End If

For Each VerbObject In mVerbs
  
  'Write the main command
  Res = RegWriteDefaultStringValue(hkeyShell, VerbObject.Verb & "\Command", VerbObject.Command)
  If Res <> ERROR_SUCCESS Then
    mLastError = "Could not write the command string" & SysErrorString(Res)
    Exit Function
  End If
  
  'If the ddeexec string is not blank write it, otherwise, delete the ddeexec key altogether
  If VerbObject.Ddeexec <> "" Then

    Res = RegWriteDefaultStringValue(hkeyShell, VerbObject.Verb & "\Ddeexec", VerbObject.Ddeexec)
    If Res <> ERROR_SUCCESS Then
      Call RegCloseKey(hkeyShell)
      mLastError = "Could not write the Ddeexec string" & SysErrorString(Res)
      Exit Function
    End If
  
    Res = RegWriteDefaultStringValue(hkeyShell, VerbObject.Verb & "\Ddeexec\Application", VerbObject.Application)
    If Res <> ERROR_SUCCESS Then
      Call RegCloseKey(hkeyShell)
      mLastError = "Could not write the Ddeexec\Application string" & SysErrorString(Res)
      Exit Function
    End If
  
    Res = RegWriteDefaultStringValue(hkeyShell, VerbObject.Verb & "\Ddeexec\Topic", VerbObject.Topic)
    If Res <> ERROR_SUCCESS Then
      Call RegCloseKey(hkeyShell)
      mLastError = "Could not write the Ddeexec\Topic string" & SysErrorString(Res)
      Exit Function
    End If
  
    'Either write ifexec or delete the key altogether. We know deleting a key can fail,
    'it may not exist
    If VerbObject.IfExecDde = "" Then
        Res = DeleteKey(hkeyShell, VerbObject.Verb & "\Ddeexec\ifexec")
        If Res <> ERROR_SUCCESS Then
            mLastError = "Could not delete the Ddeexec\IfExec key" & SysErrorString(Res)
            Exit Function
        End If
    Else
        Res = RegWriteDefaultStringValue(hkeyShell, VerbObject.Verb & "\Ddeexec\ifexec", VerbObject.IfExecDde)
        If Res <> ERROR_SUCCESS Then
            mLastError = "Could not write the Ddeexec\IfExec string" & SysErrorString(Res)
            Call RegCloseKey(hkeyShell)
        Exit Function
       End If
    End If
  Else
    Res = DeleteKey(hkeyShell, VerbObject.Verb & "\Ddeexec")
    If Res <> ERROR_SUCCESS Then
        mLastError = "Could not delete the Ddeexec key" & SysErrorString(Res)
        Exit Function
    End If
  End If

Next

Call RegCloseKey(hkeyShell)
SaveToRegistry = True

End Function
'
'Create a string which lists all the associated commands
'
Public Function Enumerate() As String

Dim lResult As String
Dim Index As Integer
Dim lVerb As ExecVerb

If Len(mClassName) = 0 Then
  Enumerate = ""
  Exit Function
End If

lResult = "File Class: " & mClassName & vbCrLf & _
            "Description: " & mDescription & vbCrLf
            
If mDefaultIconFile <> "" Then
    lResult = lResult & "Default icon is """ & mDefaultIconFile & "," & CStr(mIconIndex) & """" & vbCrLf
End If

If mDefaultVerb = "" Then
    'If the open verb exists it is the default, otherwise there is none
    On Error GoTo NoOpen
    Set lVerb = mVerbs("open")
    lResult = lResult & "Default command verb is ""open""" & vbCrLf
NoOpen:
    On Error GoTo 0
Else
  lResult = lResult & "Default command verb is """ & mDefaultVerb & """" & vbCrLf
End If

For Index = 1 To mVerbs.Count
  Set lVerb = mVerbs(Index)
  lResult = lResult & vbTab & """" & lVerb.Verb & """ command=" & lVerb.Command & vbCrLf
  If lVerb.Ddeexec <> "" Then
    lResult = lResult & vbTab & """" & lVerb.Verb & """ ddeexec=" & lVerb.Ddeexec & vbCrLf
    lResult = lResult & vbTab & vbTab & " application=" & lVerb.Application & vbCrLf
    lResult = lResult & vbTab & vbTab & " topic=" & lVerb.Topic & vbCrLf
    If lVerb.IfExecDde <> "" Then
      lResult = lResult & vbTab & vbTab & " ifexec=" & lVerb.IfExecDde & vbCrLf
    End If
  End If
Next

Enumerate = lResult

End Function
'
'Add a new verb to the collection, fail if it exists. All keys are lower case
'
Public Function AddVerb(Verb As String) As Boolean

Dim lVerb As ExecVerb
Dim lKey As String

'Get a lower case version of the key
lKey = StrConv(Verb, vbLowerCase)
On Error GoTo NotPresent
Set lVerb = mVerbs(lKey)
mLastError = "The verb you tried to add exists already"
AddVerb = False
Exit Function

NotPresent:
On Error GoTo 0

Set lVerb = New ExecVerb
lVerb.Verb = Verb
Call mVerbs.Add(lVerb, lKey)
AddVerb = True

End Function
'
'Remove a verb to the collection, fail if it does not exist
'
Public Function RemoveVerb(Verb As String) As Boolean

Dim lVerb As ExecVerb
Dim lKey As String

'Get a lower case version of the key
lKey = StrConv(Verb, vbLowerCase)

On Error GoTo NotPresent
Set lVerb = mVerbs(lKey)
Call mVerbs.Remove(lKey)
RemoveVerb = True
Exit Function

NotPresent:
On Error GoTo 0
mLastError = "The verb you tried to remove does not exist"
RemoveVerb = False
End Function
Private Sub Class_Initialize()
    Call Clear
End Sub

Private Sub Class_Terminate()
  Set mVerbs = Nothing
End Sub


