Attribute VB_Name = "modRegistry"
'
'Constants and declares for handling the Windows registry
'
Option Explicit

Public Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, ByVal lpSecurityAttributes As Long, phkResult As Long, lpdwDisposition As Long) As Long
Public Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, ByRef phkResult As Long) As Long
Public Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, ByRef lpType As Long, ByVal lpData As String, ByRef lpcbData As Long) As Long
Public Declare Function RegQueryValueExString Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Public Declare Function RegQueryValueExLong Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Long, lpcbData As Long) As Long
Public Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long
Public Declare Function RegSetValueExString Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal reserved As Long, ByVal dwType As Long, ByVal lpValue As String, ByVal cbData As Long) As Long
Public Declare Function RegSetValueExLong Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal reserved As Long, ByVal dwType As Long, lpValue As Long, ByVal cbData As Long) As Long
Public Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Public Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, ByVal cbName As Long) As Long
Public Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, ByRef lpcbValueName As Long, ByVal lpReserved As Long, ByRef lpType As Long, ByVal lpData As String, ByRef lpcbData As Long) As Long
Public Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long

'Error message output

Public Declare Function GetLastError Lib "kernel32" () As Long
Public Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, ByVal Arguments As Long) As Long

Public Const FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
Public Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200
Public Const FORMAT_MESSAGE_FROM_STRING = &H400
Public Const FORMAT_MESSAGE_FROM_HMODULE = &H800
Public Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Public Const FORMAT_MESSAGE_ARGUMENT_ARRAY = &H2000
Public Const FORMAT_MESSAGE_MAX_WIDTH_MASK = &HFF

Public Const LANG_USER_DEFAULT = &H1 'This is MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT)

' Reg Key Security Options...
Public Const SYNCHRONIZE = &H100000
Public Const READ_CONTROL = &H20000
Public Const STANDARD_RIGHTS_ALL = &H1F0000
Public Const STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
Public Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Public Const STANDARD_RIGHTS_REQUIRED = &HF0000
Public Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)

Public Const KEY_CREATE_LINK = &H20
Public Const KEY_CREATE_SUB_KEY = &H4
Public Const KEY_ENUMERATE_SUB_KEYS = &H8
Public Const KEY_EVENT = &H1
Public Const KEY_NOTIFY = &H10
Public Const KEY_QUERY_VALUE = &H1
Public Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Public Const KEY_SET_VALUE = &H2
Public Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
Public Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Public Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))


'Open/Create Options
Public Const REG_OPTION_NON_VOLATILE = 0&
Public Const REG_OPTION_VOLATILE = &H1

' Reg Key ROOT Types...
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_CONFIG = &H80000005
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_DYN_DATA = &H80000006
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const HKEY_USERS = &H80000003
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const ERROR_SUCCESS = 0
Public Const REG_SZ = 1                         ' Unicode nul terminated string
Public Const REG_DWORD = 4                      ' 32-bit number

'A string used when an item is blank
Public Const EMPTY_STRING As String = "{None}"
'
'Fill an empty string with the standard empty string, or return the original
'
Public Function DisplayString(lString As String) As String

If Len(lString) = 0 Then
    DisplayString = EMPTY_STRING
Else
    DisplayString = lString
End If

End Function
'
'The opposite of DisplayString, return empty if the standard string
'
Public Function OriginalString(lString As String) As String

If lString = EMPTY_STRING Then
    OriginalString = ""
Else
    OriginalString = lString
End If

End Function
'
'Read the default value of a key. If the key does not exist then return the system error code
'If it does but there is no default value the string is empty and the return is ERROR_SUCCESS
'
Public Function RegReadDefaultStringValue(ByVal RootKey As Long, SubKey As String, ByRef Value As String) As Long

Dim ValueLength As Long
Dim Res As Long
Dim hKey As Long

'Open the key
Res = RegOpenKeyEx(RootKey, SubKey, 0, KEY_READ, hKey)
If Res <> ERROR_SUCCESS Then
    Value = ""
    RegReadDefaultStringValue = Res
    Exit Function
End If

'The key has opened, meaning it exists, so assume all OK
RegReadDefaultStringValue = ERROR_SUCCESS

'Set the length of the result string
Value = String(512, Chr(0))
ValueLength = Len(Value)
Res = RegQueryValueEx(hKey, "", 0, REG_SZ, Value, ValueLength)
Call RegCloseKey(hKey)

'Return an empty string if the default value can't be read or is empty
If Res <> ERROR_SUCCESS Then
    Value = ""
    Exit Function
End If

'Strip off the null and after, may be different Win 9x and NT
Value = VBstring(Value)

End Function
'
'Write the default value of a key. Return the system error code
'
Public Function RegWriteDefaultStringValue(ByVal RootKey As Long, SubKey As String, ByRef Value As String) As Long

Dim Res As Long
Dim hKey As Long
Dim Disposition As Long
Dim Temp As String

'Open the key
Res = RegCreateKeyEx(RootKey, SubKey, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, hKey, Disposition)
If Res <> ERROR_SUCCESS Then
    RegWriteDefaultStringValue = Res
    Exit Function
End If

'Write the value, or remove it if empty
If Len(Value) = 0 Then
    'First determine if there is a default string value
    Temp = String(1000, Chr(0))
    Call RegReadDefaultStringValue(hKey, "", Temp)
    Temp = VBstring(Temp)
    
    'If there is one delete it, otherwise nothing to do
    If Temp <> "" Then
        Res = RegDeleteValue(hKey, "")
    Else
        Res = ERROR_SUCCESS
    End If
Else
  Res = RegSetValueExString(hKey, "", 0, REG_SZ, Value, Len(Value))
End If

Call RegCloseKey(hKey)
RegWriteDefaultStringValue = Res

End Function
'
'Enumerate and delete the key name in RootKey
'Enumeration is necessary because NT does not allow deletion of a key with sub-keys
'Return whatever error return we get from Windows
'
Public Function DeleteKey(ByVal RootKey As Long, ByVal KeyName As String) As Long

Dim KeyHandle As Long
Dim NameIndex As Long
Dim SubKeyName As String
Dim SubKeys As Collection
Dim SubKeyVar As Variant

Dim Res As Long

Res = RegOpenKeyEx(RootKey, KeyName, 0, KEY_ALL_ACCESS, KeyHandle)
If Res <> ERROR_SUCCESS Then
    DeleteKey = ERROR_SUCCESS
    Exit Function
End If

'Enumerate all the subkeys and call recursively to delete them
'We must save the names first, because RegEnumKey does not work if keys in the middle are deleted
Set SubKeys = New Collection

NameIndex = 0
Do
    SubKeyName = String(1000, Chr(0))
    Res = RegEnumKey(KeyHandle, NameIndex, SubKeyName, Len(SubKeyName))
    If Res <> ERROR_SUCCESS Then Exit Do
    SubKeyName = VBstring(SubKeyName)
    SubKeys.Add SubKeyName
    NameIndex = NameIndex + 1
Loop

For Each SubKeyVar In SubKeys
    SubKeyName = CStr(SubKeyVar)
    Res = DeleteKey(RootKey, KeyName & "\" & SubKeyName)
    If Res <> ERROR_SUCCESS Then
        Call RegCloseKey(KeyHandle)
        DeleteKey = Res
        Exit Function
    End If
Next

'Now we should be able to delete the main key
DeleteKey = RegDeleteKey(KeyHandle, "")

End Function
'
'Convert a string read by an API call to VB by cutting off the null terminator
'
Public Function VBstring(Cstring As String) As String

Dim Pos As Long

Pos = InStr(Cstring, Chr(0))
If Pos > 0 Then
    VBstring = Left(Cstring, Pos - 1)
Else
    VBstring = Cstring
End If

End Function
'
'Return a string for an error code
'
Public Function SysErrorString(ByVal Res As Long) As String

Dim SysError As String
Dim ErrBytes As Long

SysError = String(1000, Chr(0))
ErrBytes = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM Or FORMAT_MESSAGE_IGNORE_INSERTS, _
                                                0, Res, LANG_USER_DEFAULT, SysError, Len(SysError), 0)
SysError = VBstring(SysError)

If ErrBytes = 0 Then
    'If we did not get it from the system, see if we can recognise it
    If Res = 2 Then
        SysError = " System error: FILE_NOT_FOUND"
    ElseIf Res = 5 Then
        SysError = " System error: ACCESS_DENIED"
    Else
        SysError = " System Error code " & CStr(Res)
    End If
Else
    SysError = " System error: " & SysError
End If

SysErrorString = SysError

End Function
